from __future__ import unicode_literals

import hashlib
import logging
import os
import shelve

from fnmatch import fnmatch

logger = logging.getLogger(__name__)

class Database(object):
    def __init__(self, db_file, paths, ignore_files):
        self.db = shelve.open(db_file)
        self.db_file = db_file
        self.paths = paths
        self.ignore_files = [self.normalize_filename(x) for x in ignore_files]
    
    def truncate(self):
        """
        Truncates the database
        """
        logger.info('Truncated the database')
        self.db.close()
        self.db = shelve.open(self.db_file, flag='n')
    
    def rebuild(self):
        """
        Scans the paths for files and rebuilds the database.
        """
        logger.info('Rebuilding database')
        self.truncate()
        
        for root_path in self.paths:
            logger.info('Scanning %s' % root_path)
            for root, dirs, files in os.walk(root_path):
                for f in files:
                    normalized_filename = self.normalize_filename(f)
                    
                    do_skip = False
                    for ignore_file in self.ignore_files:
                        if fnmatch(normalized_filename, ignore_file):
                            do_skip = True
                            break
                    if do_skip:
                        continue

                    path = os.path.abspath(os.path.join(root, f))
                    size = os.path.getsize(path)
                    key = self.keyify(normalized_filename, size) # shelve only takes strings
                    
                    if key in self.db: # check if same file
                        old_inode = os.stat(self.db[key]).st_ino
                        new_inode = os.stat(path).st_ino
                        if old_inode != new_inode:
                            logger.warning('Duplicate key %s and %s' % (path, self.db[key]))

                    self.db[key] = path
            logger.info('Done scanning %s' % root_path)
        self.db.sync()
    
    def find_file_path(self, f, size):
        """
        Looks for a file in the database.
        """
        key = self.keyify(self.normalize_filename(f), size)

        return self.db.get(key, None)
    
    def keyify(self, name, size):
        """
        Turns a name and size into a key that can be stored in the database.
        """
        key = '%s|%s' % (size, name)
        return hashlib.sha256(key.encode('utf-8')).hexdigest()
    
    def normalize_filename(self, filename):
        """
        Normalizes a filename to better detect simlar files.
        """
        return filename.replace(' ', '_').lower()


import argparse
import logging
import os

from six.moves import configparser

from autotorrent.at import AutoTorrent
from autotorrent.db import Database
from autotorrent.clients.rtorrent import RTorrentClient

def commandline_handler():
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--config", dest="config_file", default="autotorrent.conf", help="Path to config file")
    
    parser.add_argument("-t", "--test_rtorrent", action="store_true", dest="test_rtorrent", default=False, help='Tests the connection to rTorrent')
    parser.add_argument("-r", "--rebuild", action="store_true", dest="rebuild", default=False, help='Rebuild the database')
    parser.add_argument("-a", "--addfile", dest="addfile", default=False, help='Add a new torrent file to client', nargs='+')
    parser.add_argument("-d", "--delete_torrents", action="store_true", dest="delete_torrents", default=False, help='Delete torrents when they are added to the client')
    parser.add_argument("--verbose", help="increase output verbosity", action="store_true", dest="verbose")
    
    args = parser.parse_args()
    
    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.ERROR)
    
    if not os.path.isfile(args.config_file):
        parser.error("Config file not found %r" % args.config_file)
    
    config = configparser.ConfigParser()
    config.read(args.config_file)
    
    current_path = os.getcwd()
    os.chdir(os.path.dirname(os.path.abspath(args.config_file))) # Changing directory to where the config file is.
    
    if not config.has_section('general'):
        parser.error('AutoTorrent is not properly configured, please edit %r' % args.config_file)
        quit(1)

    i = 1
    disks = []
    while config.has_option('disks', 'disk%s' % i):
        disks.append(config.get('disks', 'disk%s' % i))
        i += 1
    
    db = Database(config.get('general', 'db'), disks,
                  config.get('general', 'ignore_files').split(','))
    
    client_name = config.get('client', 'client')
    if client_name == 'rtorrent':
        client = RTorrentClient(config.get('client', 'url'),
                                config.get('client', 'label'))
    else:
        print('Unknown client %r' % client_name)
        quit(1)
    
    at = AutoTorrent(
        db,
        client,
        config.get('general', 'store_path'),
        config.getint('general', 'add_limit_size'),
        config.getfloat('general', 'add_limit_percent'),
        args.delete_torrents,
        (config.get('general', 'link_type') if config.has_option('general', 'link_type') else 'soft'),
    )
    
    if args.test_rtorrent:
        proxy_test_result = client.test_connection()
        if proxy_test_result:
            print('Connected to rTorrent successfully!')
            print('  result: %s' % proxy_test_result)
    
    at.populate_torrents_seeded()

    if args.rebuild:
        print('Rebuilding database')
        db.rebuild()
        print('Database rebuilt')

    if args.addfile:
        print('Found %s torrent(s)' % len(args.addfile))
        at.populate_torrents_seeded()
        for torrent in args.addfile:
            at.handle_torrentfile(os.path.join(current_path, torrent))

if __name__ == '__main__':
    commandline_handler()